import React, { Component } from 'react';
// import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import PropTypes from 'prop-types';
import moment from 'moment';
import InputCalendarComponent from './inputs/InputCalendarComponent';
import InputTextComponent from './inputs/InputTextComponent';
import InputDropdownComponent from './inputs/InputDropdownComponent';
import InputFileUploadComponent from './inputs/InputFileUploadComponent';
import ActionLink from './ActionLink';
import ActionButton from '../components/ActionButton';
import CustomDataTable from '../components/base/CustomDataTable';

class EditableDataTable extends Component {
	constructor(props, context) {
		super(props, context);
		this.inputTextEditor = this.inputTextEditor.bind(this);
		this.calendarEditor = this.calendarEditor.bind(this);
		this.dropdownEditor = this.dropdownEditor.bind(this);
		this.fileUploadEditor = this.fileUploadEditor.bind(this);
		this.actionTemplate = this.actionTemplate.bind(this);
		this.dateTemplate = this.dateTemplate.bind(this);
		this.objectArrayTemplate = this.objectArrayTemplate.bind(this);
		this.objectTemplate = this.objectTemplate.bind(this);
	}

	onEditorValueChange(props, value) {
		const { handleEditElement } = this.props;
		const updatedList = [...props.value];
		updatedList[props.rowIndex][props.field] = value;
		handleEditElement(props.rowData, null);
	}

	bodyTextEditor(editableFunction, props, rowData) {
		console.log(props, rowData);
		const { editable, validator } = this.props;
		return (
			<InputTextComponent
				id={props.field}
				name={props.field}
				label={props.header}
				value={rowData[props.field]}
				viewMode={editableFunction(rowData, editable) ? 'EDIT' : 'VIEW'}
				showLabel={false}
				insideTable
				validator={validator}
				validators={editableFunction(rowData, editable) ? 'required' : 'not_required'}
				onChange={(a, b, e) => this.onEditorValueChange(props, e.target.value)}
				disabled={!editableFunction(rowData, editable)}
			/>
		);
	}

	inputTextEditor(editableFunction, props) {
		console.log(editableFunction, props);
		const { editable, validator } = this.props;
		return (
			<InputTextComponent
				id={props.field}
				name={props.field}
				label={props.header}
				value={props.rowData[props.field]}
				viewMode={'EDIT'}
				showLabel={false}
				insideTable
				validator={validator}
				validators={editableFunction(props.rowData, editable) ? 'required' : 'not_required'}
				onChange={(a, b, e) => this.onEditorValueChange(props, e.target.value)}
				disabled={!editableFunction(props.rowData, editable)}
			/>
		);
	}

	calendarEditor(editableFunction, props) {
		const { editable, validator } = this.props;
		return (
			<InputCalendarComponent
				id={props.field}
				name={props.field}
				label={props.header}
				value={props.rowData[props.field]}
				onChange={(a, b, e) => this.onEditorValueChange(props, e.target.value)}
				dateFormat="dd-mm-yy"
				placeholder="dd-mm-rrrr"
				viewMode={'EDIT'}
				showLabel={false}
				showIcon
				insideTable
				validator={validator}
				validators={editableFunction(props.rowData, editable) ? 'required|date' : 'not_required'}
				disabled={!editableFunction(props.rowData, editable)}
			/>
		);
	}

	dropdownEditor(editableFunction, options, props) {
		const { editable, validator } = this.props;
		return (
			<InputDropdownComponent
				id={props.field}
				name={props.field}
				label={props.header}
				value={props.rowData[props.field]}
				options={options}
				onChange={(a, b, e) => this.onEditorValueChange(props, e.value)}
				viewMode={'EDIT'}
				valueView={props.rowData[props.field] ? props.rowData[props.field].label : ''}
				insideTable
				showLabel={false}
				validator={validator}
				disabled={!editableFunction(props.rowData, editable)}
			/>
		);
	}

	fileUploadEditor(
		editableFunction,
		field,
		fileList,
		header,
		restApiUrl,
		onUpload,
		onRemove,
		itemLabel,
		itemName,
		validators,
		viewMode,
		props
	) {
		const { editable, validator } = this.props;
		return (
			<InputFileUploadComponent
				id={field}
				name={field}
				label={header}
				fileList={props[fileList]}
				showLabel={false}
				insideTable
				restApiUrl={restApiUrl}
				multiple
				maxFileSize={1000000}
				handleUpload={this.onUpload.bind(this, props, fileList, onUpload)}
				handleRemove={this.onRemove.bind(this, props, fileList, onRemove)}
				itemLabel={itemLabel} // 'originalFileName'
				itemName={itemName} // 'fileName'
				rendered
				validator={validator}
				validators={validators}
				viewMode={editableFunction(props, editable) ? viewMode : 'VIEW'}
			/>
		);
	}

	onUpload(props, varName, onUpload, event) {
		const varValue = JSON.parse(event.xhr.response);
		const modifiedList = props[varName].concat(varValue);
		onUpload(modifiedList, props, varName);
	}

	onRemove(props, varName, onRemove, e, i) {
		const modifiedList = props[varName];
		modifiedList.splice(i, 1);
		onRemove(modifiedList, props, varName);
	}

	requiredValidator(props) {
		const value = props.rowData[props.field];
		if (value instanceof Date) {
			return moment(value).isValid();
		} else {
			return value && value.length > 0;
		}
	}

	actionTemplate(rowData) {
		const { handleEditElement, showDeleteButton } = this.props;
		if (showDeleteButton) {
			if (rowData.deleted) {
				return (
					<ActionButton
						label={'Cofnij'}
						handleClick={e => {
							e.preventDefault();
							handleEditElement(rowData, false);
						}}
						key={'remove-cancel-button'}
					/>
				);
			} else {
				return (
					<ActionButton
						label={'Usuń'}
						handleClick={e => {
							e.preventDefault();
							handleEditElement(rowData, true);
						}}
						key={'remove-button'}
					/>
				);
			}
		}
	}

	isEditable(editable) {
		return editable;
	}

	enumTemplate(field, rowData) {
		if (rowData[field] && rowData[field].label) {
			return rowData[field].label;
		} else if (rowData[field]) {
			return rowData[field];
		}
		return '';
	}

	dateTemplate(field, format, rowData) {
		if (rowData[field] && moment(rowData[field]).isValid()) {
			return moment(rowData[field]).format(format);
		} else {
			return '';
		}
	}

	dateRangeTemplate(fieldFrom, fieldTo, format, rowData) {
		let dateFrom = undefined;
		let dateTo = undefined;
		if (rowData[fieldFrom] && moment(rowData[fieldFrom]).isValid()) {
			dateFrom = moment(rowData[fieldFrom]).format(format);
		}
		if (rowData[fieldTo] && moment(rowData[fieldTo]).isValid()) {
			dateTo = moment(rowData[fieldTo]).format(format);
		}
		if (dateFrom || dateTo) {
			return `${dateFrom ? dateFrom : ''} - ${dateTo ? dateTo : ''}`;
		} else {
			return '';
		}
	}

	objectTemplate(field, objField, rowData) {
		if (rowData[field]) {
			return rowData[field][objField];
		} else {
			return '';
		}
	}

	objectJoinTemplate(fieldArray, separator, rowData) {
		if (separator === undefined) {
			separator = ', ';
		}
		const arrayOfFields = [];
		fieldArray.forEach(field => {
			if (rowData[field]) {
				arrayOfFields.push(rowData[field]);
			}
		});
		const result = arrayOfFields.join(separator);
		return result;
	}

	objectArrayTemplate(field, objField, rowData) {
		if (rowData[field] && rowData[field].map) {
			return rowData[field] && rowData[field].length > 0
				? rowData[field].map(v => v[objField]).join(', ')
				: '';
		} else {
			return '';
		}
	}

	render() {
		const {
			addButtonClassName,
			addButtonLabel,
			className,
			columns,
			editable,
			handleAddElement,
			handleSelectionChange,
			headerColumnGroup,
			id,
			label,
			paginator,
			rendered,
			selectable,
			selection,
			selectionMode,
			showAddButton,
			showLabel,
			value,
		} = this.props;
		const dynamicColumns = columns.map(col => {
			let options;
			if (col.options === null || col.options === undefined) {
				options = [];
			} else {
				options = col.options;
			}
			let editableFunc;
			if (col.editable && col.editable instanceof Function) {
				editableFunc = col.editable.bind(this, editable);
			} else {
				editableFunc = this.isEditable.bind(this, editable);
			}
			let editor;
			let body;
			if (editable) {
				if (col.editor === 'TEXT') {
					editor = this.inputTextEditor.bind(this, editableFunc);
				} else if (col.editor === 'CALENDAR') {
					editor = this.calendarEditor.bind(this, editableFunc);
					body = this.dateTemplate.bind(this, col.field, 'DD-MM-YYYY');
				} else if (col.editor === 'DROPDOWN') {
					editor = this.dropdownEditor.bind(this, editableFunc, options);
				} else if (col.editor === 'FILEULOAD') {
					let fileValidators = 'array_required';
					if (col.validators) {
						fileValidators = col.validators;
					}
					body = this.fileUploadEditor.bind(
						this,
						editableFunc,
						col.field,
						col.fileList,
						col.header,
						col.restApiUrl,
						col.onUpload,
						col.onRemove,
						col.itemLabel,
						col.itemName,
						fileValidators,
						'EDIT'
					);
				} else if (col.editor instanceof Function) {
					editor = col.editor.bind(this, editableFunc);
				} else {
					editor = col.editor;
				}
			} else {
				if (col.editor === 'CALENDAR') {
					body = this.dateTemplate.bind(this, col.field, 'DD-MM-YYYY');
				} else if (col.editor === 'FILEULOAD') {
					let fileValidators = 'array_required';
					if (col.validators) {
						fileValidators = col.validators;
					}
					body = this.fileUploadEditor.bind(
						this,
						editableFunc,
						col.field,
						col.fileList,
						col.header,
						col.restApiUrl,
						col.onUpload,
						col.onRemove,
						col.itemLabel,
						col.itemName,
						fileValidators,
						'VIEW'
					);
				}
			}
			if (col.body instanceof Function) {
				body = col.body.bind(this);
			} else if (col.body) {
				switch (col.body) {
					case 'TEXT':
						body = this.bodyTextEditor.bind(this, editableFunc, col);
						break;
				}
			}
			let validator;
			if (col.validator === 'REQUIRED') {
				validator = this.requiredValidator;
			} else {
				validator = col.validator;
			}
			return (
				<Column
					key={col.key ? col.key : col.field}
					field={col.field}
					header={col.header}
					body={body}
					className={col.className}
					sortable={col.sortable}
					editor={editor}
					editorValidator={validator}
				/>
			);
		});
		if (rendered) {
			return (
				<div className={className}>
					{showLabel ? (
						<label className="p-label" htmlFor={id} style={{ width: '100%' }}>
							{label}
						</label>
					) : null}
					<CustomDataTable
						emptyMessage="Brak rekordów do wyświetlenia"
						responsive
						value={value}
						editable={editable}
						selectable={selectable}
						selection={selection}
						selectionMode={selectionMode}
						onSelectionChange={handleSelectionChange}
						headerColumnGroup={headerColumnGroup}
						paginator={paginator}
						rows={paginator ? 5 : undefined}
						rowsPerPageOptions={[5, 10, 20]}
					>
						{dynamicColumns}
					</CustomDataTable>
					<ActionLink
						key="add-button"
						className={addButtonClassName}
						rendered={showAddButton}
						label={addButtonLabel}
						handleClick={() => {
							handleAddElement();
						}}
					/>
				</div>
			);
		} else {
			return null;
		}
	}
}

EditableDataTable.defaultProps = {
	addButtonLabel: '+ Dodaj',
	className: 'col-12',
	showLabel: false,
	editable: false,
	paginator: false,
	showAddButton: false,
	showDeleteButton: false,
	selectable: false,
	selectionMode: 'single',
	selection: {},
	rendered: true,
};

EditableDataTable.propTypes = {
	addButtonClassName: PropTypes.string,
	addButtonLabel: PropTypes.string,
	className: PropTypes.string,
	columns: PropTypes.array.isRequired,
	editable: PropTypes.bool,
	handleAddElement: PropTypes.func.isRequired,
	handleEditElement: PropTypes.func.isRequired,
	handleSelectionChange: PropTypes.func,
	headerColumnGroup: PropTypes.any,
	id: PropTypes.string.isRequired,
	label: PropTypes.string.isRequired,
	paginator: PropTypes.bool,
	rendered: PropTypes.bool,
	selectable: PropTypes.bool,
	selection: PropTypes.object,
	selectionMode: PropTypes.string,
	showAddButton: PropTypes.bool,
	showDeleteButton: PropTypes.bool,
	showLabel: PropTypes.bool,
	validator: PropTypes.object.isRequired,
	value: PropTypes.array.isRequired,
};

export default EditableDataTable;
